@@ -32,6 +32,16 @@ try:
 except ImportError:
     # Python 2.6 has no OrderedDict, use a dict instead
     OrderedDict = dict  # pylint: disable=C0103
+
+# Manage Python2/3 different modules
+# pylint: disable=F0401,E0611
+try:
+    from urllib.parse import unquote, urlparse
+except ImportError:
+    from urllib import unquote
+    from urlparse import urlparse
+# pylint: enable=F0401,E0611
+
 import re
 import xml.etree.ElementTree as ET
 
@@ -217,6 +227,7 @@ def propfind(path, xml_request, collections, user=None):
                  _tag("D", "displayname"),
                  _tag("D", "owner"),
                  _tag("D", "getetag"),
+                 _tag("ICAL", "calendar-color"),
                  _tag("CS", "getctag")]
 
     # Writing answer
@@ -331,6 +342,8 @@ def _propfind_response(path, item, props, user):
                     item.tag, item.headers, item.timezones)
             elif tag == _tag("D", "displayname"):
                 element.text = item.name
+            elif tag == _tag("ICAL", "calendar-color"):
+                element.text = item.color
             else:
                 human_tag = _tag_from_clark(tag)
                 if human_tag in collection_props:
@@ -435,9 +448,12 @@ def put(path, ical_request, collection):
     if name in (item.name for item in collection.items):
         # PUT is modifying an existing item
         collection.replace(name, ical_request)
-    else:
+    elif name:
         # PUT is adding a new item
         collection.append(name, ical_request)
+    else:
+        # PUT is replacing the whole collection
+        collection.save(ical_request)
 
 
 def report(path, xml_request, collection):
@@ -450,17 +466,18 @@ def report(path, xml_request, collection):
     root = ET.fromstring(xml_request.encode("utf8"))
 
     prop_element = root.find(_tag("D", "prop"))
-    props = [prop.tag for prop in prop_element]
+    props = [prop.tag for prop in prop_element] if prop_element else []
 
     if collection:
         if root.tag in (_tag("C", "calendar-multiget"),
                         _tag("CR", "addressbook-multiget")):
             # Read rfc4791-7.9 for info
             base_prefix = config.get("server", "base_prefix")
-            hreferences = set(
-                href_element.text[len(base_prefix):] for href_element
-                in root.findall(_tag("D", "href"))
-                if href_element.text.startswith(base_prefix))
+            hreferences = set()
+            for href_element in root.findall(_tag("D", "href")):
+                href_path = unquote(urlparse(href_element.text).path)
+                if href_path.startswith(base_prefix):
+                    hreferences.add(href_path[len(base_prefix):])
         else:
             hreferences = (path,)
         # TODO: handle other filters
@@ -469,6 +486,7 @@ def report(path, xml_request, collection):
         tag_filters = set(
             element.get("name") for element
             in root.findall(".//%s" % _tag("C", "comp-filter")))
+        tag_filters.discard('VCALENDAR')
     else:
         hreferences = ()
         tag_filters = None
